// Lrp Network Monitor: Simple network monitor written in Java
// Copyright (C) 2001 Martin Hejl
//
// This program is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation; either version 2, or (at your option)
// any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.

import java.util.Vector;
import java.awt.event.*;
import java.awt.*;

/**
 * Parent class for all views (numeric, bar, line and histogram)
 * Handles common stuff, but doesn't display any data
 *
 * @author       Martin Hejl
 * @version      20010623 0.9beta
 */
public abstract class LrpStatAbstractGraph extends java.awt.Panel implements LrpStatModelListener, ActionListener, ComponentListener {
	protected final int MINIMUM_GRID_STEP = 4;
	protected String deviceName;
	protected InterfaceInfo lastValue=null;
	protected InterfaceInfo value=null;
	protected InterfaceInfo offset=null;
	protected static final int LEGENDWIDTH = 25;
	protected int lineHeight=10;
	protected int statusLineHeight =0;
	protected int headerHeight =0;
	protected InterfaceOptions ifOpts;
	protected PopupMenu popup;
	protected Font statusFont=null;
	protected Font titleFont=null;
	protected int nMaxInSize=0;
	protected int nMaxOutSize=0;
	protected Image offScreenImg=null;
	protected Graphics myGraphics=null;
	protected long maxThroughput = 0;
	protected boolean bModelineErrorDisplayed=false;
	protected LrpStatAction action;
	protected Rectangle _clientRect = new Rectangle(0,0,0,0);
	protected boolean rectDirty = true;
	protected int nInLength=0;
	protected int nOutLength=0;
	private long lUser=-1;
	private long lSystem=-1;
	private long lIdle=-1;
	private long lNice=-1;

	private long lUser_prev=-1;
	private long lSystem_prev=-1;
	private long lIdle_prev=-1;
	private long lNice_prev=-1;

	/**
	 * Initializes the internal data and the popup-menu
	 */
	LrpStatAbstractGraph(InterfaceOptions opts) {
		// Register a component-listener, so we get informed when the
		// component is resized
		addComponentListener(this);

		deviceName = opts.deviceLabel;
		ifOpts = opts;

		// If there are actions defined, add a menu with these actions
		if (ifOpts.actions.size()>0) {

			// Create a popup menu
			popup = new PopupMenu("Actions");
			enableEvents(AWTEvent.MOUSE_EVENT_MASK);

			MenuItem item;
			CommandInfo cmd;
			// Add the specified actions to the popup menu
			for (int i=0; i<ifOpts.actions.size(); i++){
				cmd = (CommandInfo)ifOpts.actions.elementAt(i);
				item = new MenuItem(cmd.caption);
				item.setActionCommand(String.valueOf(i));
				item.addActionListener(this);
				popup.add(item);
			}
			// Add the popup menu to this component
			add(popup);

		}

		// Create an instance of Interface info with the specified offset values
		offset = new InterfaceInfo(	"", "", opts.inOffset, 0,0,0,0,0,0,0,ifOpts.outOffset,0,0,0,0,0,0,0, false, 0, 0);

		if (!ifOpts.drawTitle) headerHeight=0;  //else headerHeight += LrpStatLook.getINSETS();
		if (!ifOpts.drawStatus) statusLineHeight=0; //else statusLineHeight += LrpStatLook.getINSETS();
	}

	/**
	 * Returns a new Instance where the values are the difference of v1 and v2
	 */
	public InterfaceInfo subtract(InterfaceInfo v1, InterfaceInfo v2) {
		return(new InterfaceInfo(
			v1.getName(),
			v1.getStatus(),
			v1.getBytes(InterfaceInfo.DIRECTION_RECEIVE) - v2.getBytes(InterfaceInfo.DIRECTION_RECEIVE),
			v1.getPackets(InterfaceInfo.DIRECTION_RECEIVE) - v2.getPackets(InterfaceInfo.DIRECTION_RECEIVE),
			v1.getErrors(InterfaceInfo.DIRECTION_RECEIVE) - v2.getErrors(InterfaceInfo.DIRECTION_RECEIVE),
			v1.getDrop(InterfaceInfo.DIRECTION_RECEIVE) - v2.getDrop(InterfaceInfo.DIRECTION_RECEIVE),
			v1.getFifo(InterfaceInfo.DIRECTION_RECEIVE) - v2.getFifo(InterfaceInfo.DIRECTION_RECEIVE),
			v1.getFrame(InterfaceInfo.DIRECTION_RECEIVE) - v2.getFrame(InterfaceInfo.DIRECTION_RECEIVE),
			v1.getCompressed(InterfaceInfo.DIRECTION_RECEIVE) - v2.getCompressed(InterfaceInfo.DIRECTION_RECEIVE),
			v1.getMulticast(InterfaceInfo.DIRECTION_RECEIVE) - v2.getMulticast(InterfaceInfo.DIRECTION_RECEIVE),

			v1.getBytes(InterfaceInfo.DIRECTION_TRANSMIT) - v2.getBytes(InterfaceInfo.DIRECTION_TRANSMIT),
			v1.getPackets(InterfaceInfo.DIRECTION_TRANSMIT) - v2.getPackets(InterfaceInfo.DIRECTION_TRANSMIT),
			v1.getErrors(InterfaceInfo.DIRECTION_TRANSMIT) - v2.getErrors(InterfaceInfo.DIRECTION_TRANSMIT),
			v1.getDrop(InterfaceInfo.DIRECTION_TRANSMIT) - v2.getDrop(InterfaceInfo.DIRECTION_TRANSMIT),
			v1.getFifo(InterfaceInfo.DIRECTION_TRANSMIT) - v2.getFifo(InterfaceInfo.DIRECTION_TRANSMIT),
			v1.getCompressed(InterfaceInfo.DIRECTION_TRANSMIT) - v2.getCompressed(InterfaceInfo.DIRECTION_TRANSMIT),
			v1.getColls(InterfaceInfo.DIRECTION_TRANSMIT) - v2.getMulticast(InterfaceInfo.DIRECTION_TRANSMIT),
			v1.getCarrier(InterfaceInfo.DIRECTION_TRANSMIT) - v2.getCarrier(InterfaceInfo.DIRECTION_TRANSMIT),
			v1.getISDNDevice(),
			v1.getISDNStatus(),
			v1.getTimestamp() - v2.getTimestamp()

		));

	}

	/**
	 * Called by the model whenever new data arrives
	 * Needs to be overridden by derived classes
	 */
	public abstract void dataArrived(LrpStatModelEvent e);

	/**
	 * Returns a rectangle with the bounds of the area where the data can be displayed
	 */
	protected Rectangle getClientRect() {
		//System.err.println("INSETS=" + LrpStatLook.getINSETS)();

		if (rectDirty) {
			_clientRect.x = LrpStatLook.getINSETS();
			_clientRect.y = LrpStatLook.getINSETS()+headerHeight;
			_clientRect.width = getSize().width - 2*LrpStatLook.getINSETS();
			_clientRect.height = getSize().height-2*LrpStatLook.getINSETS()-headerHeight-statusLineHeight;

			if (_clientRect.height<0) _clientRect.height=0;
			if (_clientRect.width<0)  _clientRect.width=0;

			rectDirty = false;
		}

		return (_clientRect);
	}

	/**
	 *  Draws the header (title of the interface)
	 */
	public void drawHeader(Graphics g, String title, Color foreground) {
		if (!ifOpts.drawTitle) return;

		Font oldFont=g.getFont();

		g.setFont(titleFont);

		FontMetrics fm;
		fm = g.getFontMetrics();

		Rectangle clientRect = getClientRect();

		// Draw the title if it fits into the window
		if ((LrpStatLook.getINSETS() + fm.getLeading() + fm.getMaxAscent() + fm.getMaxDescent()) < getSize().height)  {
			if (ifOpts.displayCPUUsage) drawCPUUsage(g, clientRect);

			g.setColor(foreground);
			//g.drawString(title + " " + getSystemPercent() + "/" + getUserPercent() + "/" + getNicePercent(), LrpStatLook.getINSETS(), LrpStatLook.getINSETS() + lineHeight);
			g.drawString(title, LrpStatLook.getINSETS(), LrpStatLook.getINSETS() + lineHeight);
		}

		g.setFont(oldFont);


	}


	protected void drawCPUUsage(Graphics g, Rectangle clientRect) {

		if (ifOpts.displayCPUUsage != true) return;
		if (LrpStatLook.getINSETS()<4) return;

		Color previousColor=g.getColor();

		int sys  = (int)(getSystemPercent() * clientRect.width / 100);
		int nice = (int)(getNicePercent()   * clientRect.width / 100);
		int user = (int)(getUserPercent()   * clientRect.width / 100);
		int idle = (int)(getIdlePercent()   * clientRect.width / 100);




		int yStart = 2;
		int insets = LrpStatLook.getINSETS();

		//Debug.println(yStart + ":" + sys + "/" + user + "/" + nice + "/" + idle);
		g.setColor(LrpStatLook.getCpuIdleColor());
		if (idle>0) g.fillRect( insets+sys+user+nice,
								yStart ,
								idle,
								insets-2);

		g.setColor(LrpStatLook.getCpuNiceColor());
		if (nice>0) g.fillRect( insets+sys+user,
								yStart,
								nice,
								insets-2);

		g.setColor(LrpStatLook.getCpuUserColor());
		if (user>0) g.fillRect( insets+sys,
								yStart,
								user,
								insets-2);

		g.setColor(LrpStatLook.getCpuSystemColor());
		if (sys>0) g.fillRect( 	insets,
								yStart,
								sys,
								insets-2);


		g.setColor(previousColor);
	}

	/**
	 * Draws the footer (in-value, out-value and maximum value)
	 */
	public void drawFooter(Graphics g, long in, long out, Color foreground, long maxThroughput) {

		if (!ifOpts.drawStatus) return;

		Font oldFont=g.getFont();

		g.setFont(statusFont);

		FontMetrics fm;
		fm = g.getFontMetrics();

		// Find out how long the InValue+label is
		if (nMaxInSize==0){
			String s;
			s = "In: " + maxThroughput + " ";
			nMaxInSize = fm.stringWidth(s);
		}
		// Find out how long the OutValue+label is
		if (nMaxOutSize==0){
			String s;
			s = "Out: " + maxThroughput;
			nMaxOutSize = fm.stringWidth(s);
		}

		// Find out how long the MaxValue+label is
		int nMaxThroughputSize;
		if (ifOpts.autoScale) {
			String s;
			s = " Max:" + maxThroughput;
			nMaxThroughputSize = fm.stringWidth(s);
		} else {
			nMaxThroughputSize =0;
		}

		Rectangle clientRect = getClientRect();

		// Paint the labels, if they all fit into the control
		g.setColor(foreground);
		if (getSize().height -  (fm.getLeading() + fm.getMaxAscent() + fm.getMaxDescent()) > clientRect.y + clientRect.height
		  && nMaxInSize+nMaxOutSize+nMaxThroughputSize < clientRect.width) {
			int yPos;
			int diff = (fm.getLeading() + fm.getMaxAscent() + fm.getMaxDescent());

			yPos = clientRect.y+clientRect.height+diff;
			yPos += ((getSize().height - yPos)/2);


			String s;
			long val;
			val = in ;
			s = "In: " + Long.toString(val);
			g.drawString(s, clientRect.x, yPos);

			val = out;
			s = "Out: " + Long.toString(val);
			g.drawString(s, clientRect.x + nMaxInSize, yPos);

			if (ifOpts.autoScale) {
				s = " Max:" + maxThroughput;
				g.drawString(s, clientRect.x + nMaxInSize + nMaxThroughputSize, yPos);
			}
		}

		g.setFont(oldFont);
	}


	/**
	 * Paints the grid
	 */
	public void drawGrid(Graphics g, long maxThroughput,  InterfaceInfo value) {

		// Don't draw a grid if the device isn't online
		if (value.getISDNDevice() && value.getISDNStatus() != InterfaceInfo.ISDN_ONLINE) return;

		g.setColor(LrpStatLook.getGridColor());
		Rectangle clientRect = getClientRect();
		int gridStep;

		if (ifOpts.gridLineCount == 0 && maxThroughput != 0) {
			// if a maxthroughput was defined, transform the interval to screen coordinates
			gridStep = (int)(((ifOpts.gridInterval)*(clientRect.height))/maxThroughput);
		} else {
			if (ifOpts.gridLineCount != 0) {
				// If a linecount was defined
				gridStep = (int)(clientRect.height/ifOpts.gridLineCount);
			} else {
				// if neither a linecount nor a maxthroughput were defined, we can't draw a grid
				System.err.println("neither a linecount nor a maxthroughput were defined");
				return;
			}
		}

		// If there would be too many gridlines (because of a silly setting in GRID_INTERVAL), quit here
		if (gridStep < MINIMUM_GRID_STEP ) {
			if(!bModelineErrorDisplayed) {
				bModelineErrorDisplayed=true;
				Debug.println("Grid Interval too small - not drawing grid");
			}
			return;
		}
		for (int y=clientRect.y+clientRect.height; y>clientRect.y; y-=gridStep) {
			g.drawLine(clientRect.x,y,clientRect.x+clientRect.width-1,y);
			//for (int x=clientRect.x; x <clientRect.x+clientRect.width; x+=2) {
			//	g.drawLine(x,y,x,y);
			//}
		}
	}

	// Draw a legend in the colors of "in" and "out"
	public void drawLegend(Graphics g, Rectangle clientRect) {
		if (value.getISDNDevice() && value.getISDNStatus() != InterfaceInfo.ISDN_ONLINE) return;

		Font oldFont=g.getFont();

		g.setFont(statusFont);

		FontMetrics fm;
		fm = g.getFontMetrics();

		String strIn = "In ";
		if (nInLength==0){
			nInLength = fm.stringWidth(strIn);
		}

		String strOut = "Out";
		if (nOutLength==0){
			nOutLength = fm.stringWidth(strOut);
		}
		String strInOut = strIn + strOut;

		int xPos=clientRect.x + clientRect.width - LrpStatLook.getINSETS() - nInLength - nOutLength;
		int yPos=clientRect.y + lineHeight+2;

		// Clip the Legend if necessary
		if (yPos> clientRect.y + clientRect.height) return;

		// Clear the background around the legend

		setBackgroundColor(g, value);
		//g.setColor(LrpStatLook.getDataBackground());
		g.drawString(strInOut, xPos-1, yPos-1);
		g.drawString(strInOut, xPos-1, yPos);
		g.drawString(strInOut, xPos-1, yPos+1);
		g.drawString(strInOut, xPos, yPos+1);
		g.drawString(strInOut, xPos, yPos-1);
		g.drawString(strInOut, xPos+1, yPos-1);
		g.drawString(strInOut, xPos+1, yPos);
		g.drawString(strInOut, xPos+1, yPos+1);

		// Draw the legend for "in"
		g.setColor(LrpStatLook.getDataColor1());
		g.drawString(	strIn,
						xPos,
						yPos);

		// Clear the background around the legend
		xPos = clientRect.x + clientRect.width - LrpStatLook.getINSETS() - nOutLength;
		yPos = clientRect.y + lineHeight+2;

/*		setBackgroundColor(g,value);
		g.drawString(strOut, xPos-1, yPos-1);
		g.drawString(strOut, xPos-1, yPos);
		g.drawString(strOut, xPos-1, yPos+1);
		g.drawString(strOut, xPos, yPos+1);
		g.drawString(strOut, xPos, yPos-1);
		g.drawString(strOut, xPos+1, yPos-1);
		g.drawString(strOut, xPos+1, yPos);
		g.drawString(strOut, xPos+1, yPos+1);*/

		// Draw the legend for "out"
		g.setColor(LrpStatLook.getDataColor2());
		g.drawString(	strOut,
						xPos,
						yPos);

		g.setFont(oldFont);
	}


	/**
	 *  Does the painting that's common to all controls
	 */
	public void paint(Graphics g) {
		FontMetrics fm;

		// Set the font to be used
		Font oldFont=g.getFont();

		if (statusFont==null) {
			statusFont = new Font(LrpStatLook.getStatusFontName(), 0 , LrpStatLook.getStatusFontSize());
		}

		if (titleFont==null) {
			titleFont = new Font(LrpStatLook.getTitleFontName(), 0 , LrpStatLook.getTitleFontSize());
		}

		g.setFont(statusFont);
		fm = g.getFontMetrics();
		if (ifOpts.drawStatus) statusLineHeight = fm.getLeading() + fm.getMaxAscent() + fm.getMaxDescent()+ LrpStatLook.getINSETS();

		g.setFont(titleFont);
		fm = g.getFontMetrics();

		lineHeight = fm.getLeading() + fm.getMaxAscent() + fm.getMaxDescent();

		if (ifOpts.drawTitle) headerHeight =  lineHeight + LrpStatLook.getINSETS();


		Rectangle clientRect = getClientRect();

		// Draw the background
		g.setColor(LrpStatLook.getControlBackground());

		g.fillRect(0,0, getSize().width,getSize().height);
		if (LrpStatLook.getDrawComponentBorder()) {
			g.draw3DRect(1,1, getSize().width-2,getSize().height-2, true);
		}


		// Draw the inner border and fill the interior
		// Ok, I finally got why I always seemed to get "empty" areas (areas that would not be filled)

		// From the API-Reference:
		// Coordinates are infinitely thin and lie between the pixels of the output device. Operations which draw the outline of a
		// figure operate by traversing an infinitely thin path between pixels with a pixel-sized pen that hangs down and to the right
		// of the anchor point on the path. Operations which fill a figure operate by filling the interior of that infinitely thin path.

		// So, in short the outer bounds of fillRect(x,y,w,h) and drawRect(x,y,w,h) are different!!!

		if (LrpStatLook.getDrawClientBorder()) {
			g.draw3DRect(clientRect.x-1,
				clientRect.y-1,
				clientRect.width+2,
				clientRect.height+2,
				false);
		}

		setBackgroundColor(g,value);

		g.fillRect(clientRect.x , clientRect.y,clientRect.width+1, clientRect.height+1);

		drawHeader(g, deviceName, LrpStatLook.getControlForeground());
		g.setFont(oldFont);
	}

	public void setBackgroundColor(Graphics g, InterfaceInfo value) {
		if (value!=null) {
			if (value.getISDNDevice()) {
				switch (value.getISDNStatus()) {
					case InterfaceInfo.ISDN_OFFLINE: 	g.setColor(LrpStatLook.getDataOfflineBackground());
														break;
					case InterfaceInfo.ISDN_ONLINE: 	g.setColor(LrpStatLook.getDataOnlineBackground());
														break;
					case InterfaceInfo.ISDN_TRYING: 	g.setColor(LrpStatLook.getDataTryingBackground());
														break;

				}

			} else g.setColor(LrpStatLook.getDataOnlineBackground());
		} else g.setColor(LrpStatLook.getDataNoDevBackground());
	}

	// Don't clear the background on an uptate - this is done in paint of the implementing classes
	public void update(Graphics g) {
		paint(g);
	}

	/**
	 * Called when a mouse-event occurs
	 */
	public void processMouseEvent(MouseEvent e) {
		/**
		 * If the mouse-event was a popup-trigger (this is different on different plattforms)
		 * show the popup
		 */
		if (e.isPopupTrigger()) {
			action = new LrpStatAction();

			// Make sure the VM doesn't wait for an action thread that's blocked
			action.setDaemon(true);
			action.setPopup(popup, e.getComponent(), e.getX(), e.getY());

			action.start();
		}
		super.processMouseEvent(e);
	}

	/**
	 * Part of the actionListener interface
	 * Called, when the user selects an item from the popup menu
	 */
 	public void	actionPerformed(ActionEvent e) 	{
		int index;

		// Extract the number from the action event (to make things easier,
		// the actionCommand is the index of the command in the actions-Vector
		try {
			index = Integer.parseInt(e.getActionCommand());
			if (index>=0 && index<ifOpts.actions.size()) {

				action = new LrpStatAction();

				// Make sure the VM doesn't wait for an action thread that's blocked
				action.setDaemon(true);

				action.setCommand((CommandInfo)ifOpts.actions.elementAt(index));
				action.start();
			}
		} catch (NumberFormatException ex) {return;}
	}



	// Part of the componentListener interface
	public void componentHidden(ComponentEvent e)  {
		// Nothing to be done here
	}

	// Part of the componentListener interface
	public void componentMoved(ComponentEvent e) {
		// Nothing to be done here
	}

	// Part of the componentListener interface
	// Called when the component gets resized
	public void componentResized(ComponentEvent e) {
		// Get rid of the old offlineImage and offlineGraphics
		// since they don't have the right dimensions anymore
		// The implementing classes will create a new offscreen-image and -graphics
		// the next time their paint routine is called
			offScreenImg = null;
			myGraphics = null;

			rectDirty = true;

			repaint();

	}

	// Part of the componentListener interface
	public void componentShown(ComponentEvent e) {
		// Nothing to be done here
	}

	public Dimension getPreferredSize()	{
		return(new Dimension(20,200));
	}

	public Dimension getMinimumSize()	{
		return(new Dimension(10,500));
	}

	protected void setCPUUsage(long lSys, long lUsr, long lN, long lI) {
		long t;
		t = (lSys + lUsr + lN + lI) - (lSystem + lUser + lNice + lIdle);

		if (lSystem==-1 ||lUser==-1||lNice==-1||lIdle==-1) {

			lSystem = lSys;
			lUser = lUsr;
			lNice = lN;
			lIdle = lI;

			lSystem_prev = lSys;
			lUser_prev = lUsr;
			lNice_prev = lN;
			lIdle_prev = lI;
		}
		else
		{
			lSystem_prev = lSystem;
			lUser_prev 	= lUser;
			lNice_prev 	= lNice;;
			lIdle_prev = lIdle;;


			lSystem = lSys;
			lUser = lUsr;
			lNice = lN;
			lIdle = lI;

		}

	}

	protected long getSystemPercent(){
		long t = (lSystem + lUser + lNice + lIdle) - (lSystem_prev + lUser_prev + lNice_prev + lIdle_prev);
		if (t==0) t=1;
		return(100*(lSystem-lSystem_prev))/t;
	}

	protected long getUserPercent(){
		long t = (lSystem + lUser + lNice + lIdle) - (lSystem_prev + lUser_prev + lNice_prev + lIdle_prev);
		if (t==0) t=1;
		return(100*(lUser-lUser_prev))/t;

	}

	protected long getNicePercent(){
		long t = (lSystem + lUser + lNice + lIdle) - (lSystem_prev + lUser_prev + lNice_prev + lIdle_prev);
		if (t==0) t=1;
		return(100*(lNice-lNice_prev))/t;
	}
	protected long getIdlePercent(){
		long t = (lSystem + lUser + lNice + lIdle) - (lSystem_prev + lUser_prev + lNice_prev + lIdle_prev);
		if (t==0) t=1;
		return(100*(lIdle-lIdle_prev))/t;
	}

}