// Lrp Network Monitor: Simple network monitor written in Java
// Copyright (C) 2001 Martin Hejl
//
// This program is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation; either version 2, or (at your option)
// any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.

import java.util.Vector;
import java.awt.*;
import java.awt.event.*;


/**
 * Implements a bar graph view of the device-data
 *
 * @author       Martin Hejl
 * @version      20010623 0.9beta
 */
public class LrpStatBarGraph extends LrpStatAbstractGraph  {
	protected InterfaceInfo paintValue=null;

	LrpStatBarGraph (InterfaceOptions opts) {
		super(opts);
	}

	/**
	 * Paint the component
	 */
	public void paint(Graphics graph) {

		// Create an offscreen-image to paint on (if necessary)
		if (myGraphics == null) {
			offScreenImg = createImage(getSize().width, getSize().height);
			myGraphics = offScreenImg.getGraphics();
		}

		// Draw borders and all the other stuff that's common between all three views
		super.paint(myGraphics);


		// Compute the line-height for the current font
		FontMetrics fm;
		fm = myGraphics.getFontMetrics();
		int lineHeight = fm.getLeading() + fm.getMaxAscent() + fm.getMaxDescent();

		// If we received a null-value, print the message and exit
		if (lastValue == null || value==null) {
			myGraphics.setColor(LrpStatLook.getDataColor1());
			myGraphics.drawString("No data", 2*LrpStatLook.getINSETS(), 2*LrpStatLook.getINSETS() + 2*lineHeight);
			graph.drawImage(offScreenImg,0,0,this);
			return;
		}

		// Get the size of the area to paint on
		Rectangle clientRect = getClientRect();

		// Calculate the width of the labels
		int columnWidth = fm.stringWidth("In: ");
		if (fm.stringWidth("Out: ") > columnWidth) columnWidth = fm.stringWidth("Out: ");

		int inWidth;
		int outWidth;

		// Scale the throughput value
		inWidth = (int)((paintValue.getBytes(InterfaceInfo.DIRECTION_RECEIVE)*(clientRect.width - columnWidth - 2))/ifOpts.maxThroughput);
		outWidth= (int)((paintValue.getBytes(InterfaceInfo.DIRECTION_TRANSMIT)*(clientRect.width - columnWidth - 2))/ifOpts.maxThroughput);

		// Do some clipping - just in case the "MaxThroughput" value wasn't realistic
		if (inWidth>(clientRect.width - columnWidth - 2)) inWidth=(clientRect.width - columnWidth- 2);
		if (outWidth>(clientRect.width - columnWidth- 2)) outWidth=(clientRect.width - columnWidth- 2);

		// Draw the footer
		drawFooter(myGraphics, paintValue.getBytes(InterfaceInfo.DIRECTION_RECEIVE), paintValue.getBytes(InterfaceInfo.DIRECTION_TRANSMIT), LrpStatLook.getControlForeground(), ifOpts.maxThroughput);

		// Clip the Bars, if neccessary
		if (clientRect.y + 2 + 2*lineHeight < clientRect.y + clientRect.height) {

			// Draw the labels
			myGraphics.setColor(LrpStatLook.getDataColor1());
			myGraphics.drawString("In:", clientRect.x, clientRect.y + lineHeight+ 2);
			myGraphics.drawString("Out:",clientRect.x , clientRect.y + 2*lineHeight + 4);

			// Draw the bars
			myGraphics.setColor(LrpStatLook.getBarBackground());
			myGraphics.fillRect(clientRect.x +columnWidth, clientRect.y+2,(clientRect.width - columnWidth - 2), lineHeight);
			myGraphics.fillRect(clientRect.x +columnWidth, clientRect.y+ lineHeight +4,(clientRect.width - columnWidth- 2), lineHeight);

			myGraphics.setColor(LrpStatLook.getDataColor1());
			myGraphics.fillRect(clientRect.x +columnWidth, clientRect.y+2,inWidth, lineHeight);
			myGraphics.fillRect(clientRect.x +columnWidth, clientRect.y+ lineHeight+4,outWidth, lineHeight);
		}

		// Paint the offscreen-image to the actual graphics context
		graph.drawImage(offScreenImg,0,0,this);

	}

	/**
	 * Called by the model whenever new data arrives
	 */
	public void dataArrived(LrpStatModelEvent e) {

		// Set lastValue
		if (lastValue == null ) {
			lastValue=e.getInterfaceInfo();
		} else {
			lastValue = value;
		}

		// Save the current value
		value = e.getInterfaceInfo();

		setCPUUsage(value.getSystem(), value.getUser(), value.getNice(), value.getIdle());

		// Compute the throughput value
		paintValue = subtract(value, lastValue);
		paintValue = subtract(paintValue, offset);
		paintValue.clipNegative();

		long throughputIn  = paintValue.getBytes(InterfaceInfo.DIRECTION_RECEIVE);
		long throughputOut = paintValue.getBytes(InterfaceInfo.DIRECTION_TRANSMIT);

		// Normalize to Throughput per second
		long lTimeDiff = (value.getTimestamp() - lastValue.getTimestamp()) ;

		// Make sure we don't get a Div by zero
		if (lTimeDiff==0) lTimeDiff=1;

		throughputIn = (throughputIn * 1000) / lTimeDiff;
		throughputOut = (throughputOut * 1000) / lTimeDiff;

		paintValue.setBytes(throughputIn,InterfaceInfo.DIRECTION_RECEIVE);
		paintValue.setBytes(throughputOut,InterfaceInfo.DIRECTION_TRANSMIT);

		// Force a repaint
		repaint();
	}

}